<?php
include_once 'app/controllers/FileController.php';
require_once 'app/CSRF.php';

class MainController extends CommonFunction
{
	protected $container;

	public function __construct($container) {
		$this->container = $container;
	}

	public function main($request, $response, $args) {
		if (isset($args['params'])) {
			$params = explode('/', $args['params']);

			$data['lang'] = $params[0];

			$lang_list = $this->container->get('settings')['languages'];
			
			$is_lang = false;
			foreach ($lang_list as $lang_key => $lang_value) {
				if ($lang_value == $data['lang']) {
					$is_lang = true;
					break;
				}
			}

			if (!$is_lang) {
				$params = array_filter(array_map('trim', $params));
				if (!$params || count($params) < 1) {
					return $this->index($request, $response, $data);
				}
				$this->softRedirection($params);
				return $this->renderNotFoundPage($request, $response, null);
			}

			if ( substr($request->getUri()->getPath(), 0, 1) == '/' ) {
				$data['currentUrl'] = '//' . $request->getUri()->getHost() . $request->getUri()->getPath();
			} else {
				$data['currentUrl'] = '//' . $request->getUri()->getHost() . '/' . $request->getUri()->getPath();
			}

			if ( substr($data['currentUrl'], -1) != '/' ) {
				$data['currentUrl'] .= '/';
			}

	        $data['baseUrl'] = $this->container->get('settings')['base_url'];

	        $data['sourceDir'] = $this->container->get('settings')['source_dir'];

	        $share_url = 'https://www.hkma.gov.hk' . $request->getUri()->getPath();
	        if(substr($share_url, -1) == '/') {
			    $share_url = $share_url;
			}
			
			$data['share_url'] = $share_url;
	        $data['urlencode_url'] = urlencode( $share_url );

			// remove empty item
			$params = array_filter(array_map('trim', $params));

			$data['params'] = $params;

			$last_param = end($params);

			if (sizeof($params) == 0) {
				return $this->index($request, $response, $data);
			}

			$is_api = false;
			if ($last_param == 'api') {
				$is_api = true;
				array_pop($params);
			}

			$pathname = implode('/', $params);

			//handle coin collection programme
			if ( isset($params[sizeof($params) - 2]) && $params[sizeof($params) - 2] == 'coin-collection-programme' ){
				$district_list = $this->getDistrictList($this->container->get('settings')['source_dir'], $data['lang']);
				$district_list['all'] = 'All';
				
				if( !isset( $district_list[ end( $params ) ] ) ) {
					header('Location: '. $data['baseUrl'] . '/' . implode('/', array_slice($params, 0, sizeof($params) - 1)) );
					die();
				}

				$data['district_name'] = end($params);
				$pathname = implode('/', array_slice($params, 0, sizeof($params) - 1));
			}

			$file = $this->container->get('settings')['source_dir'] . $pathname . '.txt';

			$file_controller = new FileController($this->container);
			
			if ($file_data = $file_controller->getFilePath($file)) {
				if ($data['lang'] != $file_data['lang']) {
					$data['url_from'] = $data['currentUrl'];
					$data['url_to'] = str_replace('/' . $data['lang'], '/' . $file_data['lang'], $data['currentUrl']);
					$data['lang_from'] = $data['lang'];
					$data['lang_to'] = $file_data['lang'];

					$file_content = file_get_contents( str_replace('/' . $data['lang'], '/' . $file_data['lang'], $file) );
					$json_data = json_decode($file_content);
					if ( $json_data->template_name == 'single-monetary-statistic.php' || $json_data->template_name == 'single-daily-monetary-statistic.php' ) {
						// $new_link = str_replace('/chi/', '/eng/', $request->getUri()->getPath());
						// $new_link = str_replace('/gb_chi/', '/eng/', $new_link);
						// header('Location: ' . $new_link);
						// die();
						$file = str_replace('/chi/', '/eng/', $file);
						$file = str_replace('/gb_chi/', '/eng/', $file);
					} else {
						return $this->renderPage($response, 'redirect.php', $data);
					}
				}

				$file_content = file_get_contents($file);

				if ( $data['lang']  == 'gb_chi' ) {
					$file_content = preg_replace('/\"\\\\\/media\\\\\/chi\\\\\//', '"\/media\/gb_chi\/', $file_content);
					$file_content = preg_replace('/\"\\\\\/chi\\\\\//', '"\/gb_chi\/', $file_content);
					$file_content = preg_replace('/\"\\/media\\/chi\\//', '"\/media\/gb_chi\/', $file_content);
					$file_content = preg_replace('/\"\\/chi\\//', '"\/gb_chi\/', $file_content);
					$file_content = str_replace('滙', '汇' , $file_content);
				}

				$json_data = json_decode($file_content);

				// redirect immediately
				if (isset($json_data->global_setting) && $json_data->global_setting) {
					if ($json_data->global_setting[0]->redirect_state == 'Y' && $json_data->global_setting[1]->redirect_url != '') {
						if ($json_data->global_setting[1]->redirect_url) {
							$redirect_url = $json_data->global_setting[1]->redirect_url;

							if (substr($redirect_url, 0, 1) == '/' && substr($redirect_url, 0, 7) != '/media/') {
								$url_pieces = explode('/', $redirect_url);

								$temp_url = str_replace($url_pieces[1], $data['lang'], $redirect_url);

								if ( $temp_url != $redirect_url ) {
									$data['lang'] = $params[0];

									$data['url_from'] = $data['currentUrl'];
									$data['url_to'] = str_replace($temp_url, $redirect_url, $data['currentUrl']);
									$data['lang_from'] = $data['lang'];
									$data['lang_to'] = $url_pieces[1];

									if ( $json_data->template_name == 'single-monetary-statistic.php' || $json_data->template_name == 'single-daily-monetary-statistic.php' ) {
										// $new_link = str_replace('/chi/', '/eng/', $request->getUri()->getPath());
										// $new_link = str_replace('/gb_chi/', '/eng/', $new_link);
										// header('Location: ' . $new_link);
										// die();
									} else {
										return $this->renderPage($response, 'redirect.php', $data);
									}
								}
							}
						}
					}
				}

				$data['default_top_background'] = $this->container->get('settings')['default_top_background'];
				$data['default_image'] = $this->container->get('settings')['default_image'];
				$data['json_data'] = $json_data;
				
				$data['pathname'] = $pathname;
				$data['translation'] = $this->getTranlsation($this->container->get('settings')['source_dir'], $data['lang']);
				$data['default_settings'] = $this->getSettings($this->container->get('settings')['source_dir'], $data['lang']);
				
				if ($is_api) {
					$template_name_array = explode('-', $json_data->template_name);
					array_shift($template_name_array);
					$template = 'api-' . implode('-', $template_name_array);
				} else {
					if (isset($json_data->template_name)) {
						if ( $json_data->template_name == '' ) {
							$json_data->template_name = 'single.php';
						}
						$template = $json_data->template_name;
					} else {
						return $this->renderNotFoundPage($request, $response, $data);
					}
				}

				if (!is_file($template)) {
					return $this->renderNotFoundPage($request, $response, $data);
				} else {
					return $this->renderPage($response, $template, $data);
				}
			} else {
				$data['json_data'] = null;

				$data['default_top_background'] = $this->container->get('settings')['default_top_background'];
				$data['default_image'] = $this->container->get('settings')['default_image'];
				$data['translation'] = $this->getTranlsation($this->container->get('settings')['source_dir'], $data['lang']);
				$data['default_settings'] = $this->getSettings($this->container->get('settings')['source_dir'], $data['lang']);

				if ( in_array('daily-monetary-statistics', $params) ) {

					$params_length = 0;

					while (end($params) != 'daily-monetary-statistics') {
						array_pop($params);
						$params_length ++;
					}

					if ($params_length == 3) {
						$params_list = explode('/', $args['params'] );
						$params_list = array_filter($params_list);

	                    $temp_publish_date = str_replace('ms-', '', end($params_list));
	                    $temp_publish_date = substr_replace( $temp_publish_date, '-', '4', 0);
	                    $temp_publish_date = substr_replace( $temp_publish_date, '-', '7', 0);
	                    $data['json_data'] = (object)array();
	                    $data['json_data']->publish_date = $temp_publish_date;
	                    $data['json_data']->revision_date = date('Y-m-d');

	                    $data['no_article'] = true;

	                    return $this->renderPage($response, 'single-monetary-statistic.php', $data);
					} else {
						header('Location: ' . $data['baseUrl'] . '/' . implode('/', $params));
						exit();
					}
				} else {
					$this->softRedirection($params);
					return $this->renderNotFoundPage($request, $response, $data);
				}
			}
		} else {
			return $this->index($request, $response, '');
		}
	}

	public function faq($request, $response, $args) {
		$params = explode( '/', $request->getUri()->getPath() );
		$data['lang'] = $args['lang'];
		
		if ( substr($request->getUri()->getPath(), 0, 1) == '/' ) {
			$data['currentUrl'] = '//' . $request->getUri()->getHost() . $request->getUri()->getPath();
		} else {
			$data['currentUrl'] = '//' . $request->getUri()->getHost() . '/' . $request->getUri()->getPath();
		}

		if ( substr($data['currentUrl'], -1) != '/' ) {
			$data['currentUrl'] .= '/';
		}
		
		$uri = $request->getUri();
		$fullURL = $uri->getScheme() . '://' . $uri->getHost();
		if (!in_array($uri->getPort(), [80, 443])) {
			$fullURL .= ':' . $uri->getPort();
		}
		$fullURL .= str_replace('//' . $request->getUri()->getHost(), '', $data['currentUrl']);

		$parts = array_filter(explode('/', trim($request->getUri()->getPath(), '/')));

		// Find the position of 'frequently-asked-questions' in the segments
		$faqIndex = array_search('frequently-asked-questions', $parts);

		// Check if 'frequently-asked-questions' is found in the URL
		if ($faqIndex !== false) {
			$parts = array_slice($parts, 0, $faqIndex + 1);
			$newPath = implode('/', $parts) . '/';
			$fullURL = $uri->getScheme() . '://' . $uri->getHost();
			if (!in_array($uri->getPort(), [80, 443])) {
				$fullURL .= ':' . $uri->getPort();
			}
			$fullURL .= '/' . $newPath;
		}

		$data['origin_url'] = $fullURL;

        $data['baseUrl'] = $this->container->get('settings')['base_url'];
        $data['sourceDir'] = $this->container->get('settings')['source_dir'];
        $share_url = 'https://www.hkma.gov.hk' . $request->getUri()->getPath();
        if(substr($share_url, -1) == '/') {
		    $share_url = $share_url;
		}
		$data['share_url'] = $share_url;
        $data['urlencode_url'] = urlencode( $share_url );

		// remove empty item
		if (sizeof($params) != 0) {
			if ($params[0] == '') {
				array_shift($params);
			}
		}
		$data['params'] = $params;

		if (sizeof($params) != 0) {
			$pathname = implode('/', array_slice($params, 0, array_search('frequently-asked-questions', $data['params']) + 1, true) );
		
			$file = $this->container->get('settings')['source_dir'] . $pathname . '.txt';
			
			$file_controller = new FileController($this->container);
			
			if ($file_data = $file_controller->getFilePath($file)) {
				if ($data['lang'] != $file_data['lang']) {
					$data['url_from'] = $data['currentUrl'];
					$data['url_to'] = str_replace('/' . $data['lang'], '/' . $file_data['lang'], $data['currentUrl']);
					$data['lang_from'] = $data['lang'];
					$data['lang_to'] = $file_data['lang'];

					return $this->renderPage($response, 'redirect.php', $data);
				}

				$file_content = file_get_contents($file);

				if ( $data['lang']  == 'gb_chi' ) {
					$file_content = preg_replace('/\"\\\\\/media\\\\\/chi\\\\\//', '"\/media\/gb_chi\/', $file_content);
					$file_content = preg_replace('/\"\\\\\/chi\\\\\//', '"\/gb_chi\/', $file_content);
					$file_content = preg_replace('/\"\\/media\\/chi\\//', '"\/media\/gb_chi\/', $file_content);
					$file_content = preg_replace('/\"\\/chi\\//', '"\/gb_chi\/', $file_content);
					$file_content = str_replace('滙', '汇' , $file_content);
				}

				$json_data = json_decode($file_content);
				
				$data['default_top_background'] = $this->container->get('settings')['default_top_background'];
				$data['default_image'] = $this->container->get('settings')['default_image'];
				$data['json_data'] = $json_data;
				
				$data['pathname'] = $pathname;
				$data['translation'] = $this->getTranlsation($this->container->get('settings')['source_dir'], $data['lang']);
				$data['default_settings'] = $this->getSettings($this->container->get('settings')['source_dir'], $data['lang']);
				$data['target'] = '';
				if (isset($args['params'])) {
					$data['target'] = explode( '/', $args['params'] );
				}

				$template = $json_data->template_name;

				if ( $template ) {
					return $this->renderPage($response, $template, $data);
				}
			}
		}

		$url = $data['baseUrl'] . '/' . $data['lang'] . '/smart-consumers/frequently-asked-questions';
		return $response->withRedirect($url);
	}

	private function index($request, $response, $data) {
    	return $response->withRedirect( $this->container->get('settings')['base_url'] . '/' . $this->container->get('settings')['default_language'], 301 );
	}

	public function softRedirection($param) {
		$redirection_file = $this->container->get('settings')['source_dir'] . 'redirection.txt';
		$redirection_list = json_decode(file_get_contents($redirection_file), true);

		foreach ($redirection_list as $key => $value) {
			if ( $value[0] == $param[0]) {
				if ( substr($value[1], 0, 1) == '/' ) {
    				$data['baseUrl'] = $this->container->get('settings')['base_url'];
					header('HTTP/1.1 301 Moved Permanently');
					header('Location: ' . $data['baseUrl'] . $value[1]);
				} else {
					header('HTTP/1.1 301 Moved Permanently');
					header('Location: ' . $value[1]);
				}
				exit();
			}

			if ( $value[0] == implode('/', $param) ) {
				if ( substr($value[1], 0, 1) == '/' ) {
    				$data['baseUrl'] = $this->container->get('settings')['base_url'];
					header('HTTP/1.1 301 Moved Permanently');
					header('Location: ' . $data['baseUrl'] . $value[1]);
				} else {
					header('HTTP/1.1 301 Moved Permanently');
					header('Location: ' . $value[1]);
				}
				exit();
			}
		}
	}
}